/*******************************************************************/
/*

 *TEMP BUG FIXED 03/28/03 MS/ME									
Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

Demofile for 100 MBit Ethernet Server. 

The program transfers images from the camera to the PC. It transfers
up to 7 images (640*480 Byte) per second. Please start the program 
"image" on the camera at Telnet port 23. Then start the PC program.

After you start the camera program the camera waits at port 
SERVER_PORT (port 2002) for a string RECIEVE_BYTES (24 Bytes).

This string includes the following data:

1. x position of the image (int value, 4 Bytes)
2. y position of the image
3. dx
4. dy
5. incrx 
6. incry

If the string is complete the camera starts transfering the image.
It response the string values at port 23. Then it waits again at
port SERVER_PORT for a new string. Is the value dx=0 the program ends.

*/
/*******************************************************************/

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#define RECEIVE_BYTES     24  /* int value (4 Bytes) for x, y, dx, dy, incrx, incry */
#define SERVER_PORT     2002  /* waiting at port 2002 for any connection */
#define MEMORY              ((getvar(VPITCH)*getvar(VWIDTH)+1024)*2)  /* memory to store two images */
//#define MEMORY              ((getvar(VPITCH)*getvar(VWIDTH)+1024))  /* memory to store one image */
#define SEND_BUFFER_SIZE    (0xFFFF) //MEMORY // max size 
#define RECEIVE_BUFFER_SIZE (0xFFFF) //MEMORY // max size
/*******************************************************************/

void main(int argc, char *argv)
{
   unsigned char *linebuffer, *plinebuf, *pixel;
   int  receivedata[RECEIVE_BYTES/4]; 
   int  i, j, x, y, dx, dy, incrx, incry, result, ms, count, temp, nr=0;
   long sec;
   sockaddr_in    laddr, raddr;
   unsigned       sock, listensock;
   unsigned       error;
   uint_16        rlen;
   unsigned int opt_value;
   unsigned int opt_length = sizeof(unsigned int);


   /* transfer the image from the physical page */
   //ScrSetLogPage((int)ScrGetPhysPage);


   /* temporarily work around */
//  tpict();
//    vmode(vmLive);
//   setvar(DISP_ACTIVE,0);
   
   /* allocate memory for image raw data */
   linebuffer=(unsigned char *)sysmalloc(MEMORY+256,3);
   if((int)linebuffer==0)
   {
	 print("not enough memory\n");
	 return;
   }

   /* server services port */
   laddr.sin_family      = AF_INET;
   laddr.sin_port        = SERVER_PORT;   /* server listen at port SERVER_PORT (2002) */
   laddr.sin_addr.s_addr = INADDR_ANY;    /* every IP addr can connect to the camera */


   /* use TCP/IP protocoll and search for a free port */
   sock = socket_stream();

   if (sock == VCRT_SOCKET_ERROR) 
     {
     print("\nCreate stream socket failed");
     return;
     } 
   
#if 1  
   /* send buffer size */    
   opt_value = SEND_BUFFER_SIZE;
   error = setsockopt(sock, SOL_TCP, OPT_TBSIZE, &opt_value, opt_length);
   if (error != VCRT_OK)
   {
	 print("Could not set socket option: OPT_TBSIZE!\n");
   }
   /* receive buffer size */    
   opt_value = RECEIVE_BUFFER_SIZE;
   error = setsockopt(sock, SOL_TCP, OPT_RBSIZE, &opt_value, opt_length);
   if (error != VCRT_OK)
   {
	 print("Could not set socket option: OPT_RBSIZE!\n");
   }
#endif
#if 0  
   /* sends immediatly the data */    
   opt_value = 1;
   error = setsockopt(sock, SOL_TCP, OPT_NOSWRBUF, &opt_value, opt_length);
   if (error != VCRT_OK)
   {
	 print("Could not set socket option:  OPT_NOSWRBUF!\n");
   }
#endif
#if 0  
   /* sends immediatly the data */    
   opt_value = 10000;
   error = setsockopt(sock, SOL_TCP, OPT_RETRANSMISSION_TIMEOUT, &opt_value, opt_length);
   if (error != VCRT_OK)
   {
	 print("Could not set socket option:  OPT_RETRANSMISSION_TIMEOUT!\n");
   }
#endif

#if 0  
   /* sends immediatly the data */    
   opt_value = 1;
   error = setsockopt(sock, SOL_TCP, OPT_SEND_NOWAIT, &opt_value, opt_length);
   if (error != VCRT_OK)
   {
	 print("Could not set socket option: OPT_SEND_NOWAIT!\n");
   }
#endif

   /* MS 21.07.2009
	* with these options the connection is dropped 
	* when link fails for more than 10s !!!
	* * */
   {
	 unsigned int opt_value;
	 unsigned int opt_length = sizeof(unsigned int);

#if 0 
	 /* Set socket option: send keepalive every minute */
	 opt_value = 1; /* 1 minute keepalive interval */
	 error = setsockopt(sock, SOL_TCP, OPT_KEEPALIVE, &opt_value, opt_length);
	 if (error != VCRT_OK)
	 {
	   print("\nsetsockopt OPT_KEEPALIVE failed - 0x%x", error);
	 }
#endif	 
#if 1 
	 /* Set socket option: connection timeout after 100s without link */
	 opt_value = 100000; /* 100000ms == TCP_SENDTIMEOUT_MIN */
	 error = setsockopt(sock, SOL_TCP, OPT_CONNECT_TIMEOUT, &opt_value, opt_length);
	 if (error != VCRT_OK)
	 {
	   print("\nsetsockopt OPT_CONNECT_TIMEOUT - 0x%x", error);
	 }
#endif	 
#if 0 
	 /* Set socket option: time wait timeout */
	 opt_value = 10000; /* 10000ms => OPT_TIMEWAIT_TIMEOUT */
	 error = setsockopt(sock, SOL_TCP, OPT_TIMEWAIT_TIMEOUT, &opt_value, opt_length);
	 if (error != VCRT_OK)
	 {
	   print("\nsetsockopt OPT_TIMEWAIT_TIMEOUT - 0x%x", error);
	 }
#endif	 
   }

   /* connection between free port and selected SERVER_PORT. Now SERVER_PORT is free again for new connections. */
   error = bind(sock, &laddr, sizeof(laddr));

   if (error != VCRT_OK) 
     {
     print("\nStream bind failed - 0x%x", error);
     shutdown(sock,FLAG_CLOSE_TX); //free socket mem
	 return;
     }
 
   /* activate sock for TCP/IP connections */
   error = listen(sock, 0);

   if (error != VCRT_OK) 
     {
     print("\nListen failed - 0x%x", error);
     shutdown(sock,FLAG_CLOSE_TX); //free socket mem
     return;
     }

   listensock = sock;

   print("\n\nFast Ethernet port %d\n",laddr.sin_port);

   for (;;) {

   /* transfer the image from the physical page */
   //ScrSetLogPage((int)ScrGetPhysPage);
      /* Is there one who wants to connect to port SERVER_PORT */
      if(listensock == VCRT_selectall(0))
      {
        /* Connection requested; accept it */
        rlen = sizeof(raddr);

        /* accept connection */
        print("\nwait for accept");
        sock = accept(listensock, &raddr, &rlen);

        if (sock == VCRT_SOCKET_ERROR)
        {
          print("\nAccept failed, error 0x%x",VCRT_geterror(listensock));
          continue;
        }
        else
        {
          print("\naccept ready");
        }

        print("\nConnection from %d.%d.%d.%d, port %d",
           (raddr.sin_addr.s_addr >> 24) & 0xFF,
           (raddr.sin_addr.s_addr >> 16) & 0xFF,
           (raddr.sin_addr.s_addr >>  8) & 0xFF,
            raddr.sin_addr.s_addr        & 0xFF,
            raddr.sin_port);


        do
        {
          //print("\n\nWaiting for image size");
          count = recv(sock, (char *)receivedata, RECEIVE_BYTES, 0); 
          dx=0;

          if(count == VCRT_ERROR)
            print("\nRECV VCRT_ERROR 0x%x",VCRT_geterror(sock));
          else if(count < RECEIVE_BYTES)
               print("\nonly %d bytes read",count);
			 else if(count != RECEIVE_BYTES)
			 {
               print("\error (0x%x)",count,count);
					continue;
			 }
          else /* no errors*/
          { 
          x=receivedata[0];
          y=receivedata[1];
          dx=receivedata[2];
          dy=receivedata[3];
          incrx=receivedata[4];
          incry=receivedata[5];
          }

        //  print("\nx=%d y=%d dx=%d dy=%d incrx=%d incry=%d nr=%d", x,y,dx,dy,incrx,incry, nr++);
		  //pstr("r\n"); 

        /* dx=0 : quit program */
        if (dx)
        {
		//time_delay(10);
		//wait(0,1);  
		//tpict();  
        //ImageAssign(&a, getvar(DISP_START), DX, DY, pitch);
		if(-1==wait(IMAGE_READY,12000))
	      print("\nwait IMAGE_READY timeout!\n");
		//else
		//  print("\nwait IMAGE_READY done!\n");
		  
  		 //copy(&a,&b);
		 /* transfer the image from the physical page */
         //ScrSetLogPage(b.st);
		  
          plinebuf=linebuffer+28; /* 4 Bytes for length and 24 Bytes for image data */

          count=0;

          for(j=0;j<dy;j+=incry)
          {
            pixel=(unsigned char *)(getvar(CAPT_START)+(x+((y+j)*getvar(VPITCH))));

            for(i=0;i<dx;i+=incrx) 
            {
              *plinebuf++ = *pixel;
               pixel+=incrx;
               count++;
            }
          }

          plinebuf=linebuffer;

          count+=RECEIVE_BYTES+4; 

		  *((int *)plinebuf) = count;
		  plinebuf += 4;

          pixel=(unsigned char *)receivedata;
          for(i=0;i<RECEIVE_BYTES;i++) 
            {
              *plinebuf++ = *pixel++;
            }

          sec = getvar(SEC);
          ms  = getvar(MSEC);
          
		  //print("now send buffer count=%d\n",count); 
		  //pstr("s\n"); 
          
		  result=send(sock, (char *)linebuffer, count, 0);
          if(result==VCRT_ERROR) 
          {
            print("\nSEND VCRT_ERROR 0x%x",VCRT_geterror(sock));
          }
          else
          {
            if(result!=count)    
            print("\nSend error wrong length=%d result=%d",count,result);
          }

          sec = getvar(SEC)-sec;
          ms  = getvar(MSEC)-ms;
            
          ms = 1000*(int)sec + ms;
          //print("  t=%dms",ms);
        }
        }
        while(dx);

        shutdown(sock, FLAG_CLOSE_TX);
		sock=0;
        };

     break;

   } /* Endfor */
   shutdown(listensock, FLAG_CLOSE_TX);

	   sysfree(linebuffer);


   /* activate image display */
   setvar(DISP_ACTIVE,1);

vmode(vmLive);

} /* Endbody */


/* EOF */
